library csvparser;

class CsvParser implements Iterator<Iterator<String>> {
  List<String> _current = null;
  int _linenumber = -1;
  bool _beginningOfFile = true;
  bool get _endOfFile => _runes.isEmpty;

  Iterator<String> get current=>_current.iterator;

  int get linenumber => _linenumber;

  bool moveNext() {
    if (_endOfFile) return false;
    _current = new List<String>();
    while (true) {
      _current.add(_getNextField());
      if(_runesStartsWith(_LINEEND) || _endOfFile ) {
        _linenumber++;
        break;
      }
    }
    return true;
  }

  String get _runeString=>"runes: $_runes, length: ${_runes.length}";

  bool _validDividingRune() {
    if(_beginningOfFile) return true;
    if(_endOfFile) return true;
    return _runesStartsWith(_SEPERATOR) || _runesStartsWith(_LINEEND);
  }

  String _getNextField() {
    if(!_validDividingRune()) throw "non valid character found ${_runeString}";
    if(!_beginningOfFile) {
      if (_runesStartsWith(_SEPERATOR)) {
        _runes = _runes.skip(_SEPERATOR.length);
      } else if (_runesStartsWith(_LINEEND)) {
        _runes = _runes.skip(_LINEEND.length);
      }
    }
    _beginningOfFile=false;
    String result="";
    if (_runesStartsWith(_QUOTEMARK)) {
      _runes = _runes.skip(_QUOTEMARK.length);
      result = _inString();
      if(!_validDividingRune()) throw "bad ending to quoted field: ${_runeString}";
      return result;
    }
    while(_runes.isNotEmpty) {
      if(_validDividingRune()) break;
      if (_runesStartsWith(_QUOTEMARK))
        throw "unexpected quotemark in non-quoted field";
      result += new String.fromCharCode(_runes.first);
      _runes = _runes.skip(1);
    }
    return result;
  }

  String _inString() {
    String result="";
    while(true) {
      if(_runes.isEmpty)
        throw "unexpected end: missing end quote";
      if(_runesStartsWith(_DOUBLEQUOTEMARK)) {
        result += _QUOTEMARK.string;
        _runes = _runes.skip(_DOUBLEQUOTEMARK.length);
        continue;
      }
      if(_runesStartsWith(_QUOTEMARK)) {
        _runes = _runes.skip(_QUOTEMARK.length);
        break;
      }
      result += new String.fromCharCode(_runes.first);
      _runes = _runes.skip(1);
    }
    return result;
  }

  bool _runesStartsWith(Runes needle) {
    int needlelength = needle.length;
    if (_runes.length < needle.length) return false;
    var r = _runes.take(needlelength);
    var n = needle.take(needlelength);
    for (int i=0; i < needlelength; i++) {
      if (r.elementAt(i)!=n.elementAt(i))
        return false;
    }
    return true;
  }

  Iterable _runes;
  Runes _SEPERATOR;
  Runes _DOUBLEQUOTEMARK;
  Runes _LINEEND;
  Runes _QUOTEMARK;
  List<String> _headers;

  List<String> get headers=>_headers;

  void set headers(List<String> h) {
    _headers=h;
  }


  List<String> getLineAsList()=>_current;

  Map<String, String> getLineAsMap({List<String> headers:null}) {
    if(headers==null) headers=this._headers;
    if (headers.length != _current.length) throw "headers wrong length for current line";
    return new Map<String, String>.fromIterables(headers, _current);
  }

  CsvParser(String text, {String seperator:";", String lineend:null, String quotemark:'"', bool setHeaders:false}) {
    text = text.trim();
    _SEPERATOR = seperator.runes;
    if (lineend==null) {
      if (text.contains("\r\n"))
        lineend = "\r\n";
      else
        lineend = "\n";
    }
    while(text.startsWith(lineend))
      text = text.substring(lineend.length);
    while(text.endsWith(lineend))
      text = text.substring(0, text.length - lineend.length);
    text=text.trim();
    if(text.isEmpty) throw "file empty";
    this._runes = text.runes;
    _LINEEND = lineend.runes;
    _QUOTEMARK = quotemark.runes;
    _DOUBLEQUOTEMARK = "$quotemark$quotemark".runes;
    if(setHeaders) {
      if(this.moveNext())
        _headers = this.getLineAsList();
    }
  }

}